==> foundation-stories-0.20.2.tgz: package/stories/compound-components/03-keyboard.stories.tsx <==
import { action } from '@storybook/addon-actions';
import { Meta } from '@storybook/react';
import React, { FC, useState } from 'react';

import {
  DefaultList,
  ListSelectionProvider,
  useListSelection,
  ListStyleProvider,
  useListStyle,
  useListNavigation,
  KeyboardHandlers,
  mergeCallbacks,
  useListNavigationHandlers,
  unselect,
  Unselect,
} from '@clds/compound-components';
// eslint-disable-next-line @nrwl/nx/enforce-module-boundaries
import {
  Edit,
  Extension,
  FaceDetect,
  Settings,
  SharedCollection,
  Transform,
} from '@clds/icon';
// eslint-disable-next-line @nrwl/nx/enforce-module-boundaries
import { Item, ItemLayout, ItemProps } from '@clds/item';

import { NestedListItemsWrapper } from './common';

const ListItem: FC<ItemProps & { id: string }> = ({
  id,
  role = 'listitem',
  onClick,
  children,
  ...rest
}) => {
  const listContext = useListStyle();
  const { onClick: useListSelectionOnClick, ...selectionContext } =
    useListSelection(id);

  return (
    <Item
      data-id={id}
      role={role}
      {...listContext}
      {...selectionContext}
      {...rest}
      onClick={mergeCallbacks(onClick, useListSelectionOnClick)}
    >
      {children}
    </Item>
  );
};

const Demo = () => {
  const [selectedId, setSelectedId] = useState<string | Unselect>(unselect);
  const ids = [
    'settings',
    'transformations',
    'analyze',
    'edit',
    'extension',
    'share',
  ];

  const confirm = action('confirm');
  const cancel = action('cancel');

  const handlers: KeyboardHandlers = {
    ...useListNavigationHandlers({
      items: ids,
      activeItem: selectedId,
      onActiveItemChange: setSelectedId,
    }),
    onEnter: confirm,
    onSpace: confirm,
    onEscape: cancel,
  };
  const { focusableProps } = useListNavigation(handlers);

  return (
    <>
      <p>
        <code>selectedId: {String(selectedId)}</code>
      </p>
      <ListSelectionProvider onClick={setSelectedId} selectedId={selectedId}>
        <ListStyleProvider borderRadius="md">
          <DefaultList {...focusableProps}>
            <ListItem id={ids[0]} onClick={action('manage clicked')}>
              <ItemLayout leftSlot={<Settings />}>Manage</ItemLayout>
            </ListItem>
            <ListItem
              id={ids[1]}
              onClick={action('edit transformations clicked')}
            >
              <ItemLayout leftSlot={<Transform />}>
                Edit Transformations
              </ItemLayout>
            </ListItem>
            <ListItem id={ids[2]} onClick={action('analyze clicked')}>
              <ItemLayout leftSlot={<FaceDetect />}>Analyze</ItemLayout>
            </ListItem>
            <NestedListItemsWrapper>
              <p>
                {/* eslint-disable-next-line react/no-unescaped-entities */}
                You can see that list items don't need to be immediate children
                of List component
              </p>
              <ListItem id={ids[3]} onClick={action('edit clicked')}>
                <ItemLayout leftSlot={<Edit />}>Edit</ItemLayout>
              </ListItem>
              <ListItem id={ids[4]} onClick={action('extension clicked')}>
                <ItemLayout leftSlot={<Extension />}>Extension</ItemLayout>
              </ListItem>
            </NestedListItemsWrapper>
            <ListItem id={ids[5]} onClick={action('share clicked')}>
              <ItemLayout leftSlot={<SharedCollection />}>Share</ItemLayout>
            </ListItem>
          </DefaultList>
        </ListStyleProvider>
      </ListSelectionProvider>
    </>
  );
};

export const ConnectedKeyboard = () => {
  return (
    <>
      <h1>Adding keyboard interaction</h1>
      <p>
        Connected components demonstrated in previous stage look pretty handy
        but they still miss one important feature, keyboard navigation.
      </p>
      <p>
        We can easily add it using <strong>useListNavigation()</strong> hook
        which is responsible for listening keyboard events and firing provided
        setSelectedId callback. All you need is to insert properties returned
        from the hook into element you want to make focusable and listening for
        keyboard events. In our example we make whole list component focusable
        so returned properties are inserted into list component.
      </p>
      <p>
        Please use <kbd>up</kbd>, <kbd>down</kbd>, <kbd>home</kbd>,{' '}
        <kbd>end</kbd> keys and observe how selection changes. Please look at
        actions panel to check click or keyboard handlers.
      </p>
      <Demo />
      <h2>Drawbacks</h2>
      <p>
        {/* eslint-disable-next-line react/no-unescaped-entities */}
        Although we can easily bind keyboard navigation to our selection state,
        {/* eslint-disable-next-line react/no-unescaped-entities */}
        we can't invoke onClick property attached to each single item when the
        user presses enter key. This is because navigation logic which happens
        {/* eslint-disable-next-line react/no-unescaped-entities */}
        on parent component doesn't have access to deeply nested children
        properties. Firing item onClick prop on enter when parent component is
        focused is important to provide consistent behavior.
      </p>
    </>
  );
};

const meta: Meta<typeof Demo> = {
  title: 'Compound Components/03 Adding Keyboard',
  component: Demo,
};

export default meta;

==> foundation-stories-0.41.1.tgz: package/stories/compound-components/03-keyboard.stories.tsx <==
import { action } from '@storybook/addon-actions';
import { Meta } from '@storybook/react';
import React, { FC, useState } from 'react';

import {
  DefaultList,
  ListSelectionProvider,
  useListSelection,
  ListStyleProvider,
  useListStyle,
  useListNavigation,
  KeyboardHandlers,
  mergeCallbacks,
  useListNavigationHandlers,
  unselect,
  Unselect,
} from '@clds/compound-components';
// eslint-disable-next-line @nx/enforce-module-boundaries
import {
  Edit,
  Extension,
  FaceDetect,
  Settings,
  SharedCollection,
  Transform,
} from '@clds/icon';
// eslint-disable-next-line @nx/enforce-module-boundaries
import { Item, ItemLayout, ItemProps } from '@clds/item';

import { NestedListItemsWrapper } from './common';

const ListItem: FC<ItemProps & { id: string }> = ({
  id,
  role = 'listitem',
  onClick,
  children,
  ...rest
}) => {
  const listContext = useListStyle();
  const { onClick: useListSelectionOnClick, ...selectionContext } =
    useListSelection(id);

  return (
    <Item
      data-id={id}
      role={role}
      {...listContext}
      {...selectionContext}
      {...rest}
      onClick={mergeCallbacks(onClick, useListSelectionOnClick)}
    >
      {children}
    </Item>
  );
};

const Demo = () => {
  const [selectedId, setSelectedId] = useState<string | Unselect>(unselect);
  const ids = [
    'settings',
    'transformations',
    'analyze',
    'edit',
    'extension',
    'share',
  ];

  const confirm = action('confirm');
  const cancel = action('cancel');

  const handlers: KeyboardHandlers = {
    ...useListNavigationHandlers({
      items: ids,
      activeItem: selectedId,
      onActiveItemChange: setSelectedId,
    }),
    onEnter: confirm,
    onSpace: confirm,
    onEscape: cancel,
  };
  const { focusableProps } = useListNavigation(handlers);

  return (
    <>
      <p>
        <code>selectedId: {String(selectedId)}</code>
      </p>
      <ListSelectionProvider onClick={setSelectedId} selectedId={selectedId}>
        <ListStyleProvider borderRadius="md">
          <DefaultList {...focusableProps}>
            <ListItem id={ids[0]} onClick={action('manage clicked')}>
              <ItemLayout leftSlot={<Settings />}>Manage</ItemLayout>
            </ListItem>
            <ListItem
              id={ids[1]}
              onClick={action('edit transformations clicked')}
            >
              <ItemLayout leftSlot={<Transform />}>
                Edit Transformations
              </ItemLayout>
            </ListItem>
            <ListItem id={ids[2]} onClick={action('analyze clicked')}>
              <ItemLayout leftSlot={<FaceDetect />}>Analyze</ItemLayout>
            </ListItem>
            <NestedListItemsWrapper>
              <p>
                {/* eslint-disable-next-line react/no-unescaped-entities */}
                You can see that list items don't need to be immediate children
                of List component
              </p>
              <ListItem id={ids[3]} onClick={action('edit clicked')}>
                <ItemLayout leftSlot={<Edit />}>Edit</ItemLayout>
              </ListItem>
              <ListItem id={ids[4]} onClick={action('extension clicked')}>
                <ItemLayout leftSlot={<Extension />}>Extension</ItemLayout>
              </ListItem>
            </NestedListItemsWrapper>
            <ListItem id={ids[5]} onClick={action('share clicked')}>
              <ItemLayout leftSlot={<SharedCollection />}>Share</ItemLayout>
            </ListItem>
          </DefaultList>
        </ListStyleProvider>
      </ListSelectionProvider>
    </>
  );
};

export const ConnectedKeyboard = () => {
  return (
    <>
      <h1>Adding keyboard interaction</h1>
      <p>
        Connected components demonstrated in previous stage look pretty handy
        but they still miss one important feature, keyboard navigation.
      </p>
      <p>
        We can easily add it using <strong>useListNavigation()</strong> hook
        which is responsible for listening keyboard events and firing provided
        setSelectedId callback. All you need is to insert properties returned
        from the hook into element you want to make focusable and listening for
        keyboard events. In our example we make whole list component focusable
        so returned properties are inserted into list component.
      </p>
      <p>
        Please use <kbd>up</kbd>, <kbd>down</kbd>, <kbd>home</kbd>,{' '}
        <kbd>end</kbd> keys and observe how selection changes. Please look at
        actions panel to check click or keyboard handlers.
      </p>
      <Demo />
      <h2>Drawbacks</h2>
      <p>
        {/* eslint-disable-next-line react/no-unescaped-entities */}
        Although we can easily bind keyboard navigation to our selection state,
        {/* eslint-disable-next-line react/no-unescaped-entities */}
        we can't invoke onClick property attached to each single item when the
        user presses enter key. This is because navigation logic which happens
        {/* eslint-disable-next-line react/no-unescaped-entities */}
        on parent component doesn't have access to deeply nested children
        properties. Firing item onClick prop on enter when parent component is
        focused is important to provide consistent behavior.
      </p>
    </>
  );
};

const meta: Meta<typeof Demo> = {
  title: 'Compound Components/03 Adding Keyboard',
  component: Demo,
};

export default meta;
